<?php

namespace WHMCS\Module\Addon\Mobilpark\Lib\Events\Hooks;

use Exception;
use WHMCS\Database\Capsule;
use WHMCS\Module\Addon\Mobilpark\Config;
use WHMCS\Module\Addon\Mobilpark\Core\Engine\Factory;
use WHMCS\Module\Addon\Mobilpark\Core\Util\Tool;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Response;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Service\MPService;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Request\ProcessMessage;

Final Class ClientAdd Extends Factory
{
    use \WHMCS\Module\Addon\Mobilpark\Lib\Events\Base;

    private static $hook = "ClientAdd";
    private static $alias = "ClientAdd";
    private static $type = "client";
    private static $pattern = "Sayin {firstname} {lastname}, {companytitle} a kayit oldugunuz icin tesekkur ederiz. Email: {email} Sifre: {password}";
    private static $description = "Executes as a client is being added to WHMCS.";
    private static $variables = "{firstname},{lastname},{companytitle},{email},{password}";

    protected function hook()
    {
        return function($vars = null)
        {
            try
            {
                $replace = [];
                $report = ["status" => null, "result" => null, "userid" => null];

                /* Module settings have not been set yet. */
                if (Tool::isSettingsReady() === false) {
                    return null;
                }

                /* There are no schema details in the database. */
                if ( ! ($schema = $this->schema())) {
                    return null;
                }

                $phoneNumberField = Config::obj()->get("servicephonefield");

                $getData = Capsule::table('tblclients as c')->select('c.*', "c.id as userid");

                if (is_numeric($phoneNumberField))
                {
                    $getData->addSelect("f1.value as userphone");
                    $getData->leftJoin('tblcustomfieldsvalues as f1', function($join) use($phoneNumberField) {
                        $join->on('c.id', '=', 'f1.relid')->where('f1.fieldid', '=', Capsule::raw($phoneNumberField));
                    });
                }
                else
                {
                    $getData->addSelect("c.phonenumber as userphone");
                }

                $getData->where("c.id", $vars["userid"]);
                $getData = $getData->first();

                if ($getData) {
                    $getData = (array)$getData;
                }  else {
                    return null;
                }

                if (empty($getData["userphone"])) {
                    return null;
                }

                $numbers = preg_replace("~\D~", "", $getData["userphone"]);
                $numbers = substr($numbers, -10);

                if (strlen($numbers) <> 10) {
                    return null;
                }

                $report["userid"] = $this->getUserId($vars);

                if ( ! empty($schema->variables))
                {
                    $varArr = $this->getVariablesArray($getData, $vars);
                    $variables = explode(",", $schema->variables);

                    foreach ($variables as $variable)
                    {
                        $variableIndex = preg_replace('/[\x7B\x7D]/', '', $variable);

                        if (array_key_exists($variableIndex, $varArr)) {
                            $replace[$variable] = $varArr[$variableIndex];
                        }
                    }
                }

                $message = strtr($schema->pattern, $replace);

                $report["recipients"] = $numbers;
                $report["message"] = $message;

                $ProcessMessage = new ProcessMessage;
                $ProcessMessage->setType();
                $ProcessMessage->setSenderid();
                $ProcessMessage->setNumbers($numbers);
                $ProcessMessage->setMessage($message);

                $getProcessMessageRequest = MPService::obj()->GetProcessMessageRequest($ProcessMessage, true);

                if ( ! ($getProcessMessageRequest instanceof Response\ResponseInterface)) {
                    throw new Exception("Could not read request response.!");
                }

                $report["result"] = $getProcessMessageRequest->getRawResult();

                if ($getProcessMessageRequest->hasError()) {
                    throw new Exception($getProcessMessageRequest->getResult());
                }

                $report["status"] = 1;
                $report["external"] = $getProcessMessageRequest->getResult()["processMessage"]["message"];
            }
            catch(Exception $e)
            {
                $report["status"] = 0;
                $report["errors"] = $e->getMessage();
            }

            $this->addResponse($report);
        };
    }
}