<?php

/**
 * Mobil Park WHMCS Addon SMS Module
 *
 *
 * Kullanım Bildirimi
 *
 * Bu yazılım mobilpark.biz tarafından yaptırılmıştır.
 * Kullanım amacı mobilpark.biz'den hizmet alan, alt yapı olarak WHMCS kullanan müşterilerin
 * mobilpark.biz servisleri ile iletişiminin sağlanması, belirlenen durumlarda otomatik sms gönderiminin sağlanmasıdır.
 * WHMCS kullanan mobilpark.biz müşterileri bu yazılımı ücretsiz olarak kullanabilir, edinme amacı doğrultusunda iyileştirme yapabilirler.
 *
 *
 * Telif Hakkı Bildirimi
 *
 * Bu içerik/kod/uygulama/program/yazılım 5846 sayılı Fikir ve Sanat Eserleri Kanunu ve bu kanunla ilgili tüm yasal düzenlemeler kapsamında telif altındadır.
 * Söz konusu ürünün kısmi yada tamamı üreticisin ve dağıtıcısının açıkça belirttiği amaç dışında hiç bir şekilde farklı bir amaç için kullanılamaz/çoğaltılamaz/dağıtılamaz.
 * Aksine davrananlar hakkında ilgili mevzuat hükümleri kapsamında istisnasız gerekli yasal işlemler başlatılacaktır.
 *
 *
 * @package		WHMCS
 * @subpackage	Module
 * @category	Addon
 * @author		Mehmet Emin Yumru
 * @owner		mobilpark.biz
 * @copyright	Copyright (c) Mehmet Emin Yumru
 * @copyright	Copyright (c) mobilpark.biz
 * @contact		eminatabey@gmail.com
 * @support     https://bitbucket.org/eminmehmet/mobilpark
 * @link		https://mobilpark.biz
 * @since		Version 1.0
 * @date		2019-10-23
 *
 */

namespace WHMCS\Module\Addon\Mobilpark\Lib\Events\Hooks;

use Exception;
use WHMCS\Module\Addon\Mobilpark\Core\Engine\Factory;
use WHMCS\Module\Addon\Mobilpark\Core\Util\Tool;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Response;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Service\MPService;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Request\ProcessMessage;

Final Class CustomMessageCron Extends Factory
{
    use \WHMCS\Module\Addon\Mobilpark\Lib\Events\Base;

    /* hook and alias combine values must be uniq for all operations. */

    /* If this is not a cron event, the hook name and this must be the same. */
    private static $hook = "CustomMessageCron";

    /* filename, class, and this must be same.. */
    private static $alias = "CustomMessageCron";

    /* this only can be client or admin */
    private static $type = "admin";

    /* this only can be hook or cron. If a cron event is defined, this property must be added, otherwise not mandatory. */
    private static $operator = "cron";

    /* If a cron event is defined, this property must be added, otherwise not mandatory. */
    private static $command = "--RunCustomMessageCron";

    private static $pattern = "This is a test {companytitle}.";
    private static $description = "This event runs when the command-line operations specified in the parameters occur.";
    private static $variables = "{companytitle}";

    protected function hook()
    {
        /*
         * /opt/plesk/php/7.0/bin/php -q /var/www/vhosts/example.com/httpdocs/modules/addons/mobilpark/cron/Cron.php --RunCustomMessageCron /userid=4 /invoiceid=5
         *
         * You can run this cron with additional parameters in the format mentioned above.
         * This parameters convert to associative array. ["userid" => 4, "invoiceid" => 5]
         */
        return function($vars = null)
        {
            try
            {
                $replace = [];
                $report = ["status" => null, "result" => null, "userid" => null];

                /* Module settings have not been set yet. */
                if (Tool::isSettingsReady() === false) {
                    return null;
                }

                /* There are no schema details in the database. */
                if ( ! ($schema = $this->schema())) {
                    return null;
                }

                /* No administrator phone number is specified for this template. */
                if (empty($schema->numbers)) {
                    return null;
                }

                /* for admin 0, for client call this => $this->getUserId($vars); */
                $report["userid"] = 0;

                if ( ! empty($schema->variables))
                {
                    $varArr = $this->getVariablesArray($vars);
                    $variables = explode(",", $schema->variables);

                    foreach ($variables as $variable)
                    {
                        $variableIndex = preg_replace('/[\x7B\x7D]/', '', $variable);

                        if (array_key_exists($variableIndex, $varArr)) {
                            $replace[$variable] = $varArr[$variableIndex];
                        }
                    }
                }

                $message = strtr($schema->pattern, $replace);
                $numbers = $this->getAdminNumbers($schema->numbers);

                $report["recipients"] = $numbers;
                $report["message"] = $message;

                $ProcessMessage = new ProcessMessage;
                $ProcessMessage->setType();
                $ProcessMessage->setSenderid();
                $ProcessMessage->setNumbers($numbers);
                $ProcessMessage->setMessage($message);

                $getProcessMessageRequest = MPService::obj()->GetProcessMessageRequest($ProcessMessage, true);

                if ( ! ($getProcessMessageRequest instanceof Response\ResponseInterface)) {
                    throw new Exception("Could not read request response.!");
                }

                $report["result"] = $getProcessMessageRequest->getRawResult();

                if ($getProcessMessageRequest->hasError()) {
                    throw new Exception($getProcessMessageRequest->getResult());
                }

                $report["status"] = 1;
                $report["external"] = $getProcessMessageRequest->getResult()["processMessage"]["message"];
            }
            catch(Exception $e)
            {
                $report["status"] = 0;
                $report["errors"] = $e->getMessage();
            }

            $this->addResponse($report);

            return $report["status"];
        };
    }
}