<?php

namespace WHMCS\Module\Addon\Mobilpark\Lib\Events\Hooks;

use Exception;
use WHMCS\Config\Setting;
use WHMCS\Database\Capsule;
use WHMCS\Module\Addon\Mobilpark\Config;
use WHMCS\Module\Addon\Mobilpark\Core\Util\Tool;
use WHMCS\Module\Addon\Mobilpark\Core\Engine\Factory;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Response;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Service\MPService;
use WHMCS\Module\Addon\Mobilpark\Lib\Mobile\Request\ProcessMessage;

Final Class DomainRenewNoticeFive Extends Factory
{
    use \WHMCS\Module\Addon\Mobilpark\Lib\Events\Base;

    private static $hook = "EmailPreSend";
    private static $alias = "DomainRenewNoticeFive";
    private static $type = "client";
    private static $pattern = "Sayin {firstname} {lastname}, {domain} alanadiniz {expirydate} ({x} gun sonra) tarihinde sona erecektir. Hizmetiniz kapanmadan önce hemen yenileyin!";
    private static $description = "This is executed before the fifth domain renewal mail is sent.";
    private static $variables = "{firstname},{lastname},{domain},{expirydate},{x}";

    protected function hook()
    {
        return function ($vars = null) {
            try
            {
                if ( ! in_array($vars["messagename"], ["Expired Domain Notice", "Upcoming Domain Renewal Notice"])) {
                    return null;
                }

                $DomainRenewalNoticeConfig = Setting::getValue("DomainRenewalNotices");
                $DomainRenewalNoticeConfig = explode(",", preg_replace("/\s+/", "", $DomainRenewalNoticeConfig));
                $DomainRenewalNoticeConfig = array_values(array_filter($DomainRenewalNoticeConfig, function($value) {
                    return (is_numeric($value) && ($value > 0 || $value < 0));
                }));

                if (count($DomainRenewalNoticeConfig) < 5 || $vars["mergefields"]["domain_days_until_nextdue"] != $DomainRenewalNoticeConfig[4]) {
                    return null;
                }

                $replace = [];
                $report = ["status" => null, "result" => null, "userid" => null];

                /* Module settings have not been set yet. */
                if (Tool::isSettingsReady() === false) {
                    return null;
                }

                /* There are no schema details in the database. */
                if ( ! ($schema = $this->schema())) {
                    return null;
                }

                $phoneNumberField = Config::obj()->get("servicephonefield");

                $getData = Capsule::table('tblclients as c')->select('c.*', "c.id as userid");

                if (is_numeric($phoneNumberField))
                {
                    $getData->addSelect("f1.value as userphone");
                    $getData->leftJoin('tblcustomfieldsvalues as f1', function ($join) use ($phoneNumberField) {
                        $join->on('c.id', '=', 'f1.relid')->where('f1.fieldid', '=', Capsule::raw($phoneNumberField));
                    });
                } else {
                    $getData->addSelect("c.phonenumber as userphone");
                }

                $getData->where("c.id", $vars["mergefields"]["client_id"]);
                $getData = $getData->first();

                if ($getData) {
                    $getData = (array)$getData;
                } else {
                    return null;
                }

                if (empty($getData["userphone"])) {
                    return null;
                }

                $numbers = preg_replace("~\D~", "", $getData["userphone"]);
                $numbers = substr($numbers, -10);

                if (strlen($numbers) <> 10) {
                    return null;
                }

                $domainData = Capsule::table("tbldomains as d")->select("d.*", "d.id as domainid")->where("id", $vars["mergefields"]["domain_id"])->first();
                $domainData = (array)$domainData;
                $report["userid"] = $vars["mergefields"]["client_id"];

                if (!empty($schema->variables))
                {
                    $varArr = $this->getVariablesArray($getData, $domainData, ["x" => $vars["mergefields"]["domain_days_until_nextdue"]]);
                    $variables = explode(",", $schema->variables);

                    foreach ($variables as $variable)
                    {
                        $variableIndex = preg_replace('/[\x7B\x7D]/', '', $variable);

                        if (array_key_exists($variableIndex, $varArr))
                        {
                            $replace[$variable] = $varArr[$variableIndex];
                        }
                    }
                }

                $message = strtr($schema->pattern, $replace);

                $report["recipients"] = $numbers;
                $report["message"] = $message;

                $ProcessMessage = new ProcessMessage;
                $ProcessMessage->setType();
                $ProcessMessage->setSenderid();
                $ProcessMessage->setNumbers($numbers);
                $ProcessMessage->setMessage($message);

                $getProcessMessageRequest = MPService::obj()->GetProcessMessageRequest($ProcessMessage, true);

                if (!($getProcessMessageRequest instanceof Response\ResponseInterface))
                {
                    throw new Exception("Could not read request response.!");
                }

                $report["result"] = $getProcessMessageRequest->getRawResult();

                if ($getProcessMessageRequest->hasError())
                {
                    throw new Exception($getProcessMessageRequest->getResult());
                }

                $report["status"] = 1;
                $report["external"] = $getProcessMessageRequest->getResult()["processMessage"]["message"];
            }
            catch (Exception $e)
            {
                $report["status"] = 0;
                $report["errors"] = $e->getMessage();
            }

            $this->addResponse($report);
        };
    }
}